
;
; Util.tf 
; Miscellaneous utility macros.
;
; Required by most of my tf programs.
; Questions, comments, criticisms to Symbolic.
;

/set util_tf=8
/require lisp.tf

/def -i uDef = /def -i -- u%{*}
/uDef Ver = /echo @(#) util.tf  v0.8  (05/18/96)

;
; Test if a macro exists
;    (/isMacro doesn't like macros that begin with a dash)
;
; To use:
;
; /if /uMacroExist (macro) %; /then \
;     macro exists %;\
;  /else \
;     macro doesn't exist %;\
;  /endif

/uDef MacroExist = /test "$(/list -s -i -- %{*})"!~""

;
; Action Queue
;
; Right now this is set up for Overdrive, however, this could be easily
; changed to work on any mud, even more than one mud by checking
; WORLDNAME.
;
; Trigs off of the SATIATION: display.  Executes macros from a queue
; each time it is triggered, or simulates output if queue is empty.
;
; To use:
;
;  /uSyncCmd (macro to be executed)
;
; or:
; 
;  /uaPutQ (macro to be executed)
;  send uActCmd to mud
;

/set uActCmd=,

/def -i -p100 -mregexp -ag -t'^[> ]*SATIAT' uActionTrig = \
   /let action=$(/uaGetQ) %;\
   /if /test action=~"" %; /then \
       /echo %* %;\
    /else \
       /eval /%{action} %;\
    /endif

/uDef aGetQ = /uGetQ uActionQ
/uDef aPutQ = /uPutQ %{1} uActionQ

/uDef SyncCmd = \
   /uaPutQ %{1} %;\
   /send %{uActCmd}

;
; Gagging
;
;  /uGag (mud commands)
;  /uGagOn  - turn gagging of all mud output on
;  /uGagOff - turn gagging of all mud output off
;
; For best results, only use mud commands strung together with \10.
; The first and last commands may NOT be tf commands.
;
; Note gag is priority zero, so any triggered text WILL be displayed
; through the gag.
;

/uDef Gag = \
   /let uGagSuffix= %;\
   /if /test "%{1}"=~"-s" %; /then \
       /let uGagSuffix=%{2} %;\
       /shift 2 %;\
    /endif %;\
   /uaPutQ uGagOn %;\
   /uaPutQ uGagOff %;\
   /if /test uGagSuffix!~"" %; /then \
       /eval %{uActCmd}\10%{*}\10%{uActCmd}\10%{uGagSuffix} %;\
    /else \
       /eval %{uActCmd}\10%{*}\10%{uActCmd} %;\
    /endif

/uDef GagOn = \
   /uGagOff %;\
   /def -i -p0 -ag -t'*' uGag2

/uDef GagOff = \
   /purge -i uGag2

;
; Stack/Queue functions
;
;  /uPush value (stackname)  - Push value onto stackname
;  /uPop (stackname)         - Pull value from stackname
;  /uPutQ value (queuename)  - Put value into queue
;  /uGetQ (queuename)        - Pull value from queuename
;  /uClear (stack/queuename) - Clear stack/queue
;
; From stack-q.tf, modified a bit for my personal taste
;

/uDef Push = /eval /set %{2}=%1 %%{%{2}}

/uDef Pop = /eval /car %%{%{1}}%%; /set %{1}=$$(/cdr %%{%{1}})

/uDef PutQ = /eval /set %{2}=%%{%{2}} %1

/uDef GetQ = /uPop %{1}

/uDef Clear = /set %{1}=

;
; Hook calling macros
;
;  /uCallHook macroname   - Executes macroname if macro is defined and not 
;                            empty.
;  /uCallVarHook variable - If variable exists, does a /uCallHook %{variable}
;
; Used to prevent warnings with newer versions of tf.
;

/uDef CallHook = \
   /if /uMacroExist %{1} %; /then \
       /let uHook=${%{1}} %;\
       /if /test uHook !~ "" %; /then \
           /%{1} %;\
        /endif %;\
    /endif 

/uDef CallVarHook = \
   /if /test strlen(%{1}) %; /then \
       /eval /uCallHook %%{%{1}} %;\
    /endif

;
; Time Conversion (seconds -> mm:ss)
;

/uDef PrintTime = \
   /let time=%{1} %;\
   /let minutes=$[time/60] %;\
   /let seconds=$[time-(minutes*60)] %;\
   /if /test seconds<10 %; /then \
       /let seconds=$[strcat("0",seconds)] %;\
    /endif %;\
   /echo %{minutes}:%{seconds}

;
; Cheap FP Division routine
;
; /uDiv (decimal places) (numerator) (denominator)
;
; Added bonus of printing - instead of giving an error message on a 
; zero denominator.
;

/uDef Div = \
   /if /test %{3} <= 0 %; /then \
       /echo -- - %;\
       /break %;\
    /endif %;\
   /set factor=1 %;\
   /let places=%{1} %;\
   /for i 1 %{places} /set factor=$$[factor*10] %;\
   /let num=%{2} %;\
   /let denom=%{3} %;\
   /set result=$[num*factor/denom] %;\
   /let len=$[strlen(result)] %;\
   /if /test len > places %; /then \
       /echo $[substr(result,0,len-places)].$[substr(result,len-places)] %;\
    /else \
      /for i 0 places-len /set result=$$[strcat(0,result)] %;\
      /echo .%{result} %;\
    /endif %;\
   /unset result %;\
   /unset factor

;
; Formatting macros
;
; Can't find any other alternative that works decent in older versions
; of tf.
;

/uDef Fr = \
   /if ({#} == 2) \
       /echo :$[strcat(strrep(" ",{2}-strlen({1})),{1})] %;\
    /else \
       /echo :$[strrep(" ",{1})] %;\
    /endif
   
/uDef Fl = \
   /if ({#} == 2) \
       /echo $[strcat({1},strrep(" ",{2}-strlen({1})))]: %;\
    /else \
       /echo $[strrep(" ",{1})]: %;\
    /endif

;
; Better mapcar command
;
; To use:
;
; /ForEach (listname) (command)
;
; command _may_ contain spaces.
;

/def -i ForEach = \
   /eval /set list=%%{%{1}} %;\
   /shift %;\
   /let cmd=%{*} %;\
   /while /test $(/length %{list}) %; /do \
          /eval %{cmd} $(/car %{list}) %;\
          /eval /set list=$(/cdr %{list}) %;\
    /done

; Just in case it's needed!

/test uSpace := substr('a a',1,1)


