;; General trigger framework for building bot behavior
;; Expects the following variables to be set (from world file, presumably):
;;   TankName, ThugName, HealerName, MageName
loaded bot.tf
;;/require lisp.tf
;;/require badprompt.tf

;; countList can be called as a function, and will return the number of space-delimited
;; sub-arguments in the arguments
;; e.g. countList('one', 'two three') returns 3
/def -i countList = /return $(/length %{*})

/def -i ecar = /return '$(/car %{*})'
/def -i ecdr = /return '$(/cdr %{*})'

;; defines a macro called %{1}Hook that calls a list of listeners, passing %2 arguments to each
;; List of listeners is %{1}Listeners (global variable)
;; To trigger all listeners, call /call%{1}Listeners and pass the appropriate number of args (%2 args)
;; To define a hook that doesn't check argument count, omit or provide -1 for %2
;; e.g. /defineListenerHook mobDied 1
;;      Now, register listeners by appending macro names to %mobDiedListeners
;;      /registerListener mobDied lootCorpse
;;		You've defined /lootCorpse elsewhere, and it takes one argument
;;      /callmobDiedListeners beggar
;;      This alerts all listeners that "beggar" has just died
;;      All registered macros will be called with one argument 
;;		> /lootCorpse beggar
;; 
/def defineListenerHook = \
	/let argList=%; \
	/if ({2} = -1) \
		/let argList=$[strcat(argList,",%%{","*","}")]%; \
	/else \
		/let i=1%; \
		/while (i <= {2}) /let argList=$[strcat(argList,",%%{",i+1,"}")]%; /let i=$[i + 1]%; /done%; \
	/endif%; \
	/if ({#} == 1) /defineListenerHook %{1} -1%;/return%;/endif%; \
	/def -i %{1}Hook = \
		/if (countList({1}) > 0) \
			/test $$[ecar({1})]($[substr(argList,1)])%%; \
			/test %{1}Hook(ecdr({1})%argList)%%; \
		/endif%; \
	/let argList=%; \
	/if ({2} = -1) \
		/let argList=$[strcat(argList,"%%{","*","}")]%; \
	/else \
		/let i=1%; \
		/while (i < {2}) \
			/let argList=$[strcat(argList,"%%{",i,"},")]%; \
			/let i=$[i + 1]%; \
		/done%; \
		/let argList=$[strcat(argList,"%%{",{2},"}")]%; \
	/endif%; \
	/def -i call%{1}Listeners = \
		/if (%{2} == -1 | %{2} == {#}) \
			/test %{1}Hook(%{1}Listeners,%argList)%%; \
		/else \
			/echo The %{1} hook takes %{2} parameters; you passed %%{#}%%; \
		/endif%; \
	/set %{1}Listeners=

;; Convenience method: /callListeners Foo === /callFooListeners
/def callListeners = /call%{1}Listeners %{-1}

/def appendToList = /eval /set %{1} %%{%{1}} %{2}

;; Registers a macro named %2 to listen for the event %1
;; e.g. /registerListener mobDied lootCorpse
;; Registers the lootCorpse macro to be called when the mobDied event is triggered
;; (that call is /callmobDiedListeners; see above)
/def registerListener = \
	/deregisterListener %1 %2%; \
	/if (${%{1}Hook} !~ '') /appendToList %{1}Listeners %{2}%; \
	/else /echo No hook named %{1} has been defined%; \
	/endif

/def deregisterListener = /eval /set %{1}Listeners=$$(/remove %{2} %%{%{1}Listeners})
/def unregisterListener = /deregisterListener %*


/defineListenerHook mobDied 0
/def -mglob -t'* died.' bhMobDied1 = /callMobDiedListeners
/def -mglob -t'* is no longer living.' bhMobDied2 = /callMobDiedListeners
/def -mglob -t'* has perished.' bhMobDied3 = /callMobDiedListeners
/def -mglob -t'* says good-bye to life.' bhMobDied4 = /callMobDiedListeners
/def -mglob -t'* croaked.' bhMobDied5 = /callMobDiedListeners
/def -mglob -t'* cries out for {*} mommy... and falls deathly silent.' bhMobDied6 = /callMobDiedListeners
/def -mglob -t'* soul has spiraled into the ether.' bhMobDied7 = /callMobDiedListeners
/def -mglob -t'* has bit the dust.' bhMobDied8 = /callMobDiedListeners
/def -mglob -t'* soul has passed on to the great beyond.' bhMobDied9 = /callMobDiedListeners
/def -mglob -t'* body remains, but the life has gone.' bhMobDied10 = /callMobDiedListeners
/def -mglob -t'* has breathed {*} last.' bhMobDied11 = /callMobDiedListeners
/def -mglob -t'* has left the realms of the living.' bhMobDied12 = /callMobDiedListeners
/def -mglob -t'* died!' bhMobDied13 = /callMobDiedListeners
/def -mglob -t'* falls over dead.' bhMobDied14 = /callMobDiedListeners
/def -mglob -t'The ghost utters one last wail and vanishes into darkness.' bhMobDied15 = /callMobDiedListeners
/def -mglob -t'* is an ex-*' bhMobDied16 = /callMobDiedListeners
/def -mglob -t'* has succumbed to {*} injuries.' bhMobDied17 = /callMobDiedListeners
/def -mglob -t"* would rather be living, but isn't." bhMobDied18 = /callMobDiedListeners
/def -mglob -t'* met {*} end.' bhMobDied19 = /callMobDiedListeners
/def -mglob -t'* isn\'t just dead, but completely dead.' bhMobDied20 = /callMobDiedListeners
/def -mglob -t'* has given up on life.' bhMobDied21 = /callMobDiedListeners
/def -mglob -t'* has departed from this life.' bhMobDied22 = /callMobDiedListeners
/def -mglob -t'* has passed away.' bhMobDied23 = /callMobDiedListeners
/def -mglob -t'* has checked out.' bhMobDied24 = /callMobDiedListeners
/def -mglob -t'* is no more.' bhMobDied25 = /callMobDiedListeners
/def -mglob -t'* demon turns to vapor as it dies, leaving a pile of empty armor.' bhMobDied26 = /callMobDiedListeners
/def -mglob -t'The demon turns to vapor as it dies.' bhMobDied27 = /callMobDiedListeners
/def -mglob -t'* explodes into bits of fine powder.' bhMobDied28 = /callMobDiedListeners
/eval /def -mregexp -t"^(%TankName|%HealerName|%MageName|%ThugName) has sent Khifayl to join the stars he so dearly loves.$$" bhMobDied29 = \
	/callMobDiedListeners

;; HP/SP tracking
/defineListenerHook hpDisplay 4
/defineListenerHook hpChange 1
/defineListenerHook spChange 1
/def -i -q -mregexp  -t'^-=> HP: ([0-9]*)/([0-9]*).+ ([0-9]*)/([0-9]*).* <=-' hpDisplayTrig = \
	/callHpDisplayListeners %P1 %P2 %P3 %P4
/def -i -mregexp -t'^HP : ([0-9]+) \(([0-9]+)\)  [A-Z]+ : ([0-9]+) \(([0-9]+)\)' hpQueryTrig = \
	/callHpDisplayListeners %P1 %P2 %P3 %P4

/def -i recordHpSp = \
	/eval /set dhp $$[%1 - $[world_info()]CurrHp]%; \
	/eval /set dsp $$[%3 - $[world_info()]CurrSp]%; \
	/set $[world_info()]CurrHp=%1%; \
	/set $[world_info()]CurrSp=%3%; \
	/set $[world_info()]MaxHp=%2%; \
	/set $[world_info()]MaxSp=%4%; \
	/if (dhp != 0) /callHpChangeListeners %dhp%; /endif%; \
	/if (dsp != 0) /callSpChangeListeners %dsp%; /endif

/registerListener hpDisplay recordHpSp

/defineListenerHook intoxication 2
/def -i -q -mregexp -t'SATIATION : ([^,]+), INTOXICATION : ' intoxDisplayTrig = \
	/set $[world_info()]Satiation=%P1%; \
	/set $[world_info()]Intoxication=%PR%; \
	/test callIntoxicationListeners('%P1','%PR')

/def -t'You are completely sober.' soberedUp = ,

/def -i -q -mregexp -t'^XP : ([0-9]+)  \(([0-9]+) left to spend\), GD : ([0-9]+)$' xpDisplayTrig = \
	/set $[world_info()]XP %P1%; \
	/set $[world_info()]XPToSpend %P2%; \
	/set $[world_info()]Money %P3

;; Enchant tracking!
;; blur, stoneskin, etc.
;; TODO: add bless
/defineListenerHook enchantChange 2
/def -mglob -t'You invoke the stoneskin spell on yourself*' stoneskinOnMe = \
	/callEnchantChangeListeners Stoneskin 1
/def -mglob -t'* casts a spell, and your skin thickens...' stoneskinOn = \
	/callEnchantChangeListeners Stoneskin 1
/def -mglob -t'Your skin feels like * to your touch.' stoneskinReport = \
	/callEnchantChangeListeners Stoneskin 1
/def -mglob -t'Your skin feels less weird*' stoneskinWoreOff = \
	/callEnchantChangeListeners Stoneskin 0
/def -mglob -t'Your thick skin takes the blow, then becomes normal again*' stoneskinAbsorb = \
	/callEnchantChangeListeners Stoneskin 0
/def -mglob -t'The blow penetrates somewhat your magical skin, which vanishes*' stoneskinPartialAbsorb = \
	/callEnchantChangeListeners Stoneskin 0
/def -msimple -t'You invoke the corona spell on yourself.' coronaOn = /callEnchantChangeListeners Corona 1
/def -mglob -t'Your skin is surrounded by a * corona.' coronaReport = /callEnchantChangeListeners Corona 1
/def -mglob -t'The corona blazes brightly, then fades away*' coronaAbsorb = \
	/callEnchantChangeListeners Corona 0
/def -mglob -t'Your protective corona vanishes*' coronaWoreOff = \
	/callEnchantChangeListeners Corona 0
/def -mglob -t'The corona flares up, then abruptly vanishes*' coronaPartialAbsorb = \
	/callEnchantChangeListeners Corona 0
/def -mglob -t"'dispel blur' will end the enchantment." blurOn = /callEnchantChangeListeners Blur 1
/def -mglob -t'Suddenly, the world comes back into focus*' blurOff = /callEnchantChangeListeners Blur 0
/def -msimple -t'You blink and blink and blink, and the haze vanishes.' blurDispel = /callEnchantChangeListeners Blur 0
/def -mglob -t'The world is {*} blurry.' blurReport = /callEnchantChangeListeners Blur 1
/def -msimple -t'Your whole body tingles.' invisOn = /callEnchantChangeListeners Invis 1
/def -msimple -t'You tingle and shimmer...the invisibility spell has worn off.' invisOff = /callEnchantChangeListeners Invis 0
/def -msimple -t'You feel invisible.' invisReport = /callEnchantChangeListeners Invis 1
/def -msimple -t'You are surrounded by a magical force.' armorReport = /callEnchantChangeListeners Armor 1
;;/def -mglob -t'* casts a spell, and a field of {*} light surrounds you!' armorOn = /callEnchantChangeListeners Armor 1
/def -msimple -t'The aura may be dispelled to end the enchantment.' armorOn2 = /callEnchantChangeListeners Armor 1
/def -msimple -t'With a minor focus of your will, the armor aura dissipates.' armorDispel = /callEnchantChangeListeners Armor 0
/def -msimple -t'Your protective armor aura fails...' armorOff = /callEnchantChangeListeners Armor 0
/def -msimple -t'The aegis may be dispelled to end the enchantment.' aegisOn = /callEnchantChangeListeners Aegis 1
/def -msimple -t'You are surrounded by a magical aegis.' aegisReport = /callEnchantChangeListeners Aegis 1
/def -msimple -t'Your protective aegis fails...' aegisOff = /callEnchantChangeListeners Aegis 0
/def -msimple -t'With a minor focus of your will, the aegis dissipates.' aegisDispel = /callEnchantChangeListeners Aegis 0
/def -msimple -t'You may dispel the field at any time.' auraOn = /callEnchantChangeListeners Aura 1
/def -msimple -t'A sparkling aura surrounds you.' auraReport = /callEnchantChangeListeners Aura 1
/def -msimple -t'The sparkling field floats away on a breeze.' auraOff = /callEnchantChangeListeners Aura 0
/def -msimple -t'Your bark-like skin returns to normal.' barkskinOff = /callEnchantChangeListeners Barkskin 0
/def -mglob -t"A humble request to Llardin to 'dispel' the enchantment will cause your skin*" barkskinOn = \
	/callEnchantChangeListeners Barkskin 1
/def -msimple -t"Your skin is thick and barklike." barkskinReport = /callEnchantChangeListeners Barkskin 1
/def -msimple -t"You feel your lungs twinge." waterbreathOn = /callEnchantChangeListeners Waterbreath 1
/def -msimple -t"Your lungs feel funny, almost like you could breathe under water." waterbreathReport = /callEnchantChangeListeners Waterbreath 1
/def -msimple -t"You feel the water-breathing wear off." waterbreathOff = /callEnchantChangeListeners Waterbreath 0
/def -msimple -t"Your lungs return to their air-breathing state." waterbreathDispel = /callEnchantChangeListeners Waterbreath 0

/def -mglob -t"* makes the sign of a hex, and you stagger * as you feel your limbs turn to jello!" jinxed = /callEnchantChangeListeners Jinx 1
/def -mglob -t"You are * clumsy." jinxReport = /callEnchantChangeListeners Jinx 1
/def -msimple -p9 -t"Your muscles firm up, and you feel much steadier on your feet." unjinxed = /callEnchantChangeListeners Jinx 0

;/def -msimple -t'You are unable to resist the vine...you struggle to free yourself.' entangled = /callEnchantChangeListeners Entangle 1
;/def -msimple -t'You struggle against the bonds, but fail to break free.' stillEntangled = /callEnchantChangeListeners Entangle 1
;; Hack incoming.. some problem with muffle settings prevents Earth from seeing his own message for breaking free
/eval /def -wThug -msimple -t'The vine holds!  %TankName is held fast!' tankEntangled = /repeat -wTank -S 1 /callEnchantChangeListeners Entangle 1
/eval /def -wThug -msimple -t'%TankName struggles against the bonds, but fails to break free.' tankStillEntangled = /repeat -wTank -S 1 /callEnchantChangeListeners Entangle 1
/eval /def -wThug -msimple -t'%TankName suddenly bursts its leafy bonds!' tankUnentangled = /repeat -wTank -S 1 /callEnchantChangeListeners Entangle 0

;;/def echoHpSp = /if (fg_world() =~ world_info()) /echo - -=> HP: %1/%2  SP: %3/%4 <=-%;/endif
;;/registerListener hpDisplay echoHpSp

;;/def -mregexp -t'<< ([A-Z][a-z]*) >> (.*)' partycmd = %P2

/def formParty = \
	/thug join %TankName%;/mage join %TankName%;/healer join %TankName%; \
	/tank accept %ThugName%;/tank accept %MageName%;/tank accept %HealerName


