;; This file describes a bot that will be configured with a list of targets.
;; The bot will run around to each target and kill it if it's present.
;; If a circuit is completed with no living targets, it will go to the
;; Cath and wait for a minute.

;; Assumes a 4-character party: Tank, Thug, Healer, Mage.  Thug will lead.
;; Expects Tank to be a fighter (sigil trigs)
;; Expected aliases:
;; alias k kill *

/loaded bot2.tf
/require bot.tf
/require muck.tf
/require makepath.tf

/mpgag off

;; important variables

;; Spells cast each round of combat (leave blank for weapons/wands)
/set priestSpell=
/set mageSpell=

;; For automatically buying alcohol (assuming booze shop is pathable)
/set alcoholToBuy=jug of amontillado
/set alcohol=empty
/set container=chest
/set moneycontainer=bag

;; botting is the master flag to turn the main bot (the running around part) on and off
/set botting 0
;; debuff controls casting of jinx and faerie
/set debuff 1

;; These are used by the bot, and hopefully don't need to be adjusted
;; However, regrouping and inCombat tend to be error-prone
;; regrouping indicates whether all chars are together and following or not
/set regrouping 0
/set inCombat 0
/set ready 1
;; waits tracks the number of times the bot has run out of targets and waited for reset
/set waits 0
/set targetAcquired 0

/def -ag -hBGTRIG bgTrigGag

;; autocast controls all automatic actions
;; if it's off, no buffs will be cast, no bashing, no cure, etc
/set autocast 1
;; So if it's off, warn the user of the dumb bot situation
/def -msimple -h'PROMPT > ' catchprompt = \
	/if (!autocast) \
		/prompt -aCred DUMB>%; \
	/else \
		/prompt >%; \
	/endif

/def -mglob -p9 -t'You have died.' iamdead = \
	/if (botting) \
		/set botting 0%; \
		/repeat -wThug -S 1 /goto cath%; \
		/repeat -2 1 pray%; \
	/endif

/def setEnchantStatus = /eval /set $[world_info()]%{1} %{2}
/registerListener enchantChange setEnchantStatus

;; If out of mana, run away to the staging room and wait
/def oom = \
	/set targetAcquired 0%; \
	/set inCombat 0%; \
	/repeat -wThug -S 1 /goto staging%; \
	/healer ,%; /mage ,%; \
	/set ready 0

/defineListenerHook tick
;; This was experimental, letting the MUD handle tick timing instead of TF
/def -i -msimple -ag -t'There is no help available on tick.' helpTick = \
	/let currTime $[time()]%; \
	/if (mudTickTime) \
		/let instTickVariance $[currTime - mudTickTime - 2]%; \
		/set cumulativeTickVariance $[cumulativeTickVariance + instTickVariance]%; \
		/echo *** Tick Variance (instantaneous/cumulative): %instTickVariance / %cumulativeTickVariance%; \
	/else \
		/set cumulativeTickVariance 0%; \
	/endif%; \
	/set mudTickTime %currTime
;; To use the MUD timing, set up an alias (in OD) like 'alias tick do help tick, tick'
;; then have goTick call the alias (send to mud) instead of calling /repeat
;;/def -i goTick = /repeat -2 i /calltickListeners

;; Char-specific trigs
;; These tf files define the Action macros called below, and anything they need
;; e.g. cure trigs are in healer.tf
/load thug.tf
/load healer.tf
/load mage.tf
/load tank.tf
/def -i goHealer = /repeat -wHealer -2 i /if (autocast) /HealerAction%%; /endif
/def -i goMage = /repeat -wMage -2 i /if (autocast) /MageAction%%; /endif
/def -i goThug = /repeat -wThug -2 i /if (autocast) /ThugAction%%; /endif
/def -i goTank = /repeat -wTank -2 i /if (autocast) /TankAction%%; /endif

;; Call actions a bit after the tick, to make sure everything gets resolved in the right order
/def startBots = /goTick%;/repeat -0.5 1 /goHealer%%;/goMage%%;/goThug%%;/goTank
;; Start the ticks and actions a bit after receiving an hpdisplay, for timing
/def -p19 -F -1 -mregexp -t'^-=> HP: ([0-9]*)/([0-9]*).+ ([0-9]*)/([0-9]*).* <=-' autoStartBots = \
	/repeat -0.5 1 /startBots%%; /echo Starting bot processes!

;; Booze handling follows

;; Make use of boozle as much as possible when restoring SP
/def getHealerSp = \
	/if ((alcohol =~ '' | alcohol =~ 'empty' | HealerCurrSp > HealerMaxSp * 0.6) & HealerIntoxLevel = 0) \
		/healer boozle me%;/healer ,%; \
	/elseif (alcohol !~ '' & alcohol !~ 'empty' & (!regrouping | HealerIsHere) & HealerCurrSp < HealerMaxSp * 0.6) \
		/healer guzzle %{alcohol}%;/healer ,%; \
	/endif
/def getMageSp = \
	/if ((alcohol =~ '' | alcohol =~ 'empty' | MageCurrSp > MageCurrSp * 0.6) \
			& MageIntoxLevel = 0 \
			& (!regrouping | (HealerIsHere & MageIsHere))) \
		/healer boozle %MageName%; /mage ,%; \
	/elseif (alcohol !~ '' & alcohol !~ 'empty' & (!regrouping | MageIsHere) & MageCurrSp < MageMaxSp * 0.6) \
		/mage guzzle %{alcohol}%;/mage ,%; \
	/else /mage ,%; \
	/endif

/def -wThug -mglob -t'*finishes off the *!' alcoholEmpty = \
	recycle %alcohol%; \
	/set alcohol empty

/def -wThug -mregexp -t'^Brother Cadmus gives you a ([a-z]+).$' gotAlcohol = /set alcohol %P1%; keep %P1
/def -wThug -mglob -t'You check your pockets, but do not have enough money.' nocash = /unset alcohol
/def -wThug -mglob -t"Brother Cadmus says:  You can't carry that*" noroom = drop 5000 coins%;get %{alcohol}

/def readyCheck = \
	/set ready 0

/def getSp = /getHealerSp%;/getMageSp

/def intoxeval = \
		/if (%{2} =~ 'Sober') /intoxresult 0%; \
		/elseif (%{2} =~ 'Buzzed') /intoxresult 1%; \
		/elseif (%{2} =~ 'Sloshed') /intoxresult 2%; \
		/elseif (%{2} =~ 'Wasted') /intoxresult 3%; \
		/elseif (%{2} =~ 'Plastered') /intoxresult 4%; \
		/elseif (%{2} =~ 'Totally smashed') /intoxresult 5%; \
		/else /intoxresult 99%; \
		/endif
/def intoxresult = /set $[world_info()]IntoxLevel %{1}
/registerListener intoxication intoxeval


;; Targeting follows


/mpdef oldstaging =
/mpdef oldtarget =

;; '/target foo' sets you up to kill 'foo', assuming thats a target you've configured
/def target = \
	/eval /echo Switching target from %{target} to: %{1}%; \
	/eval /mpdef oldtarget = %%{%{target}Loc}%; \
	/eval /mpdef oldstaging = %%{%{target}SafeLoc}%; \
	/eval /mpdef target = %%{%{1}Loc}%; \
	/eval /mpdef staging = %%{%{1}SafeLoc}%; \
	/eval /set targetName=%%{%{1}TargetName}%; \
	/eval /set chargeTarget=%%{%{1}IsChargeTarget}%; \
	/eval /set chargeDir=%%{%{1}ChargeDir}%; \
	/set target %{1}
	
;; Switches to the next target in your list
/def targetNext = \
	/if (world_info() =~ 'Thug') \
		/set targetAcquired 0%; \
		/set inCombat 0%; \
		/if (botting) \
			/if (TankEntangle | HealerEntangle | MageEntangle | ThugEntangle) \
				/repeat -wThug -2 1 /targetNext%; \
				/return%; \
			/endif%; \
			/goto staging%; \
			/eval /teardown%{target}%; \
			/let tgt=$[ecar(targetList)]%; \
			/target %tgt%; \
			/set targetList=$[ecdr(targetList)] %tgt%; \
			/goto staging from oldstaging%; \
			/eval /setup%{tgt}%; \
			/set ready 0%; \
			/set startTime $[time()]%; \
			/set startThugXP %ThugXPToSpend%; \
		/else /goto cath%; \
		/endif%; \
	/endif

;; Handles target cleanup, resetting for next target
/def deadTarget = \
	..%; \
	/if (world_info() =~ 'Thug') \
		/kill %reenterPID%; \
		/set targetJinx 0%; \
		/set targetFaerie 0%; \
		/set sd 0%; \
		/set targetAcquired 0%; \
		/set inCombat 0%; \
		/if (botting) \
			/set killedTarget 1%; \
			/mage ,%; \
			/healer ,%; \
			get all from corpse%; put all in %container%; \
			/repeat -wThug -0.5 1 \
				/if (!(TankEntangle | HealerEntangle | MageEntangle | ThugEntangle)) \
					/goto staging%%; \
				/endif%%; \
				/set ready 0%; \
		/endif%; \
	/endif
/registerListener mobDied deadTarget

;; bh stands for "bot hook".. didn't stick to that naming convention too much, obviously
/def bhkill = /repeat -wTank -2 1 /eval sigil %%targetName

;; Tank's sigil is how we know if the target is there
/def -wTank -mglob -t'Your squire says:  Pardon me, Sir, but the {*} is not here.' missingTarget = /handleMissingTarget
/def -wTank -mglob -t"{*} says:  Um, Sir, someone's already claimed this." claimedTarget = /handleMissingTarget
/def -wTank -msimple -t"Your squire tells you:  Um, Sir, I'm already watching ...mumble...." alreadySigiled = \
	recall%; sigil %targetName
/def -wTank -mglob -t"Your squire says:  Yes, Sir!  I'll watch *" liveTarget = \
	/set targetAcquired 1%; \
	/checkDebuff

;; If rejinx is set, require a notably jinx before starting combat
/def checkDebuff = \
	/if (targetAcquired) \
		/if (debuff & !(targetJinx >= 4 | (targetJinx & !rejinx))) /repeat -wTank -2 1 /checkDebuff%; \
		/else /startCombat%; \
		/endif%; \
	/endif

/defineListenerHook inCombat 1

;; Sets up for start of combat, issues 'kill' commands.  set icCallout 1 to call out in party chat
/def ic = \
	/if (!autocast) \
		/echo -aCred *****************%; \
		/echo -aCred **** WARNING ****%; \
		/echo -aCred * Autocast OFF! *%; \
		/echo -aCred *****************%; \
	/endif%; \
	/set targetName %*%; \
	/set targetJinx 0%; \
	/set targetFaerie 0%; \
	/set inCombat 1%; \
	/all k %*%; \
	/test callinCombatListeners({*})%; \
	/if (ThugWeapon =/ '*{spear|hammer|flail|staff|driftwood}*') \
		/thug remove shield\10wield %ThugWeapon%; \
	/endif%; \
	/if (icCallout) p ic %*%; /endif%; \
	/if (bash) /checkbash%; /endif

;; Initiates combat, including CHARGE! if configured, and making sure Tank is tanking
/def startCombat = \
	/if (!chargeTarget | !(charge =~ 'ready' & berserk !~ 'active')) \
		/tank k %targetName%; \
		/eval /set healerKillPID $$[repeat('-wHealer -2 1 /eval k %%targetName')]%; \
		/eval /set mageKillPID $$[repeat('-wMage -4 1 /eval k %%targetName')]%; \
		/eval /set thugKillPID $$[repeat('-wThug -6 1 /eval k %%targetName')]%; \
		/eval /set reenterPID $$[repeat('-wTank -2 1 /eval k %%targetName%%;/goto staging%%;/goto target%%;/set inCombat 1%%;/callInCombatListeners %targetName%%;/set targetAcquired 0')]%; \
	/else \
		/repeat -wThug -S 1 /goto staging%; \
		/repeat -wTank -1 1 /goto target%%;k %targetName%%;/goto staging%; \
		/eval /set healerKillPID $$[repeat('-wHealer -6 1 /eval k %%targetName')]%; \
		/eval /set mageKillPID $$[repeat('-wMage -6 1 /eval k %%targetName')]%; \
		/eval /set thugKillPID $$[repeat('-wThug -3 1 CHARGE! %%chargeDir')]%; \
		/eval /set reenterPID $$[repeat('-6 1 /where target%%; /set inCombat 1%%; /callInCombatListeners %targetName%%;/set targetAcquired 0')]%; \
	/endif
	

/def -wTank -mglob -t'No * here !' noTarget = recall%; /kill %reenterPID%; /handleMissingTarget

/def handleMissingTarget = \
	/set targetAcquired 0%; \
	/set inCombat 0%; \
	/kill %healerKillPID%; \
	/kill %thugKillPID%; \
	/kill %mageKillPID%; \
	/if (botting) /repeat -wThug -1 1 /doneWithTarget%; /endif

/def -wTank -mglob -t'* squire stares at you in disgust.' sigiledTarget = /missingTarget

;; End of target business, whether missing or killed
;; Checks for a complete circuit with no targets
/def doneWithTarget = \
	/if (regrouping) /set inCombat 0%; /followAll%; /repeat -wThug -2 1 /goto staging%; \
	/elseif (killedTarget) \
		/set killedTarget 0%; /set lastKilledTarget %target%; \
		/calculateMVP %target%; \
		/repeat -wThug -1 1 /targetNext%; \
	/elseif (target !~ lastKilledTarget) /repeat -wThug -1 1 /targetNext%; \
	/else /repeat -2 3 /set inCombat 0%; /goto cath%; /set botting 0%; \
		/repeat -wThug -0:2 1 \
			/set botting 1%%; \
			/targetNext%; \
		/set waits $[waits+1]%; \
	/endif

/def calculateMVP = \
	/let dTime=$[time() - startTime]%; \
	/let dXP=$[ThugXPToSpend - startThugXP]%; \
	/if (dXP > 0) \
		/eval /set timeSpent%{1} $$[timeSpent%{1} + dTime]%; \
		/eval /set xpGained%{1} $$[xpGained%{1} + dXP]%; \
	/endif

/def mvp = \
	/showMvp %targetList

/def showMvp = \
	/eval /echo %{1}: $$[xpGained%{1} / (timeSpent%{1} / 3600.0)] XP/hr%; \
	/if (%{#} > 1) /showMvp %{-1}%; /endif	


;; Gates and doors close on their own--watch out!
;; This tries to keep the party together, but is a little error-prone
/eval /def -t'%ThugName has lost you!' brokenFollow = \
	/set $$[world_info()]IsHere 0%%; \
	/set $$[world_info()]Following 0%%; \
	/if (!regrouping) \
		/set targetAcquired 0%%; \
		/set inCombat 0%%; \
		/set regrouping 1%%; \
		/if (botting) \
			/repeat -wThug -S 1 /goto staging%%; \
			/set ready 0%%; \
		/endif%%; \
	/endif

/eval /def -t'%ThugName is now your leader.' fixedFollow = \
	/set $$[world_info()]Following 1

/def -mglob -t'Alas! That you cannot follow!' cantFollow = \
	/set $[world_info()]Following 0%; \
	/if (botting) /brokenFollow%; /endif

/eval /def -mglob -t'You have been summoned by %MageName!' summoned = \
	/set $$[world_info()]IsHere 1

/def -wMage -msimple -t'Everyone in your party is here!' allHere = \
	/set HealerIsHere 1%; \
	/set TankIsHere 1%; \
	/set MageIsHere 1

/eval /def -wThug -mglob -t'%MageName appears in a flash of light!' teleportedTo = \
	/set MageIsHere 1

;/def -wHealer -t'Cure whom?' missingHealTarget = /brokenFollow

/def -w'Tank' -mglob -t'* demon bellows:  Prepare to die,*' dgSwitch1 = /dgSwitch
/def -w'Tank' -mglob -t'* demon guardian steps out of a rift in space.' dgSwitch2 = /dgSwitch
/def -wTank -mglob -t'In the branches of the Hanging Tree (down)' inTree = /if (botting) d%;s%; /endif
/def dgSwitch = /if (inCombat) /if (botting) /goto staging%; /goto target%; /else climb tree%; /endif%; /endif

;; If the city is pathable, we'll automatically donate when our container is full
;; We'll try to keep all chars' XP equal by choosing the lowest XP char to donate
/def -mglob -t'* There is no more room.' containerFull = /set containerFull 1
	
/def donateStuff = \
	/if ({#}) \
		/let lowestXPChar=%1%; \
	/else \
		/let lowestXP=$[HealerXP]%; \
		/let lowestXPChar=Healer%; \
		/if (MageXP < lowestXP) /let lowestXP=%MageXP%; /let lowestXPChar=Mage%; /endif%; \
		/if (ThugXP < lowestXP) /let lowestXP=%ThugXP%; /let lowestXPChar=Thug%; /endif%; \
		/if (TankXP < lowestXP) /let lowestXP=%TankXP%; /let lowestXPChar=Tank%; /endif%; \
	/endif%; \
	/def -wThug -mglob -t"{*} can't carry any more." donateTooHeavy = \
		/if ({#} > 1) \
			/shift%%; \
			/donateStuff %{*}%%; \
		/elseif ({#} = 0) \
			/donateStuff Healer Tank Mage Thug%%; \
		/endif%; \
	/def -wThug -msimple -t"Ok." donateGiven = \
		/repeat -w%{lowestXPChar} -S 1 \
			donate all from %container%%%; ..%%%; give %container to %ThugName%%; \
		/undef donateTooHeavy%%; \
		/undef donateGiven%; \
	/if (lowestXPChar =~ 'Thug') \
		/donateGiven%; \
	/else \
		/thug give %container to $[world_info(lowestXPChar, 'character')]%; \
	/endif

/def -wThug -mglob -t'*(open) kept.' doneDonating = /kill %donatePID%; /set containerFull 0%; /set donating 0%; put all in %container

/def handleMoney = \
	put 4000 coins in %moneycontainer%;g%;g%;g%;g%; \
	drop 4000 coins%;g%;g%;g%;g

;; Farmers!  This isn't really the place for this, but it was my first botting target, so it's here.
/def setupFarmers = \
	/echo Setting up for Farmers%; \
	/unregisterListener mobDied deadTarget%; \
	/registerListener mobDied nextFarmer%; \
	/def nextFarmer = \
		/if (world_info() =~ 'Thug') \
			get hat from corpse%%;put hat in bag%%;get hat%%;/bhkill farmer%%;/repeat -wTank -3 1 se%%%;nw%%; \
		/endif
/def killFarmers = /repeat -wThug -S 1 /nextFarmer
/def teardownFarmers = \
	/echo Done with Farmers%; \
	/unregisterListener mobDied nextFarmer%; \
	/registerListener mobDied deadTarget%; \
	/undef nextFarmer
/set FarmersLoc ->Viking 2 n nw
/set FarmersSafeLoc ->Viking 2 n


;; Define targets!
;; See target files for examples
/def defineSimpleTarget = \
	/set %{1}SafeLoc %{2}%; \
	/set %{1}Loc %{2} %{3}%; \
	/if ({#} = 4 & {4} !~ '') \
		/set %{1}TargetName %{4}%; \
	/else \
		/set %{1}TargetName %{1}%; \
	/endif%; \
	/def kill%{1} = /eval /bhkill %%{targetName}%; \
	/set targetList %{targetList} %{1}%; \
	/set lastKilledTarget %{1}

/def defineChargeTarget = \
	/test defineSimpleTarget({1}, {2}, {3}, {4})%; \
	/set %{1}IsChargeTarget 1%; \
	/set %{1}ChargeDir %{3}

;; Does this work?
/def killSingle = \
	/set botting 0%; \
	/set tempTargetName %1%; \
	/set tempSafeLoc %2%; \
	/set tempLoc=%; \
	/target temp%; \
	/where target%; \
	/bhkill

;; Since we're controlling all chars from one socket most of the time,
;; we need a way to see what the other chars' see for tell and class line
/def -mregexp -t'^[A-Z][a-z]* (tells you|replies):' echoTell = \
	/if (fg_world() !~ world_info()) \
		/if (emulation =~ 'ansi_attr') /echo -aBCmagenta [$[world_info()]] %P0%PR%; \
		/else /echo [$[world_info()]] %P0%PR%; \
		/endif%; \
	/endif
/def -mregexp -t'^\*-\*' echoClass = \
	/if (fg_world() !~ world_info() \
		    & !((fg_world() =~ 'Thug' & world_info() =~ 'Tank') \
			| (fg_world() =~ 'Tank' & world_info() =~ 'Thug'))) \
		/if (emulation =~ 'ansi_attr') /echo -aCmagenta [$[world_info()]] %P0%PR%; \
		/else /echo [$[world_info()]] %P0%PR%; \
		/endif%; \
	/endif

;; In case you want more resistance, I guess?  I think I used this for the snow phoenix's decimate.  Tank already wore one, and mage has corona.
;; uncloak is unfinished, obviously
/def cloak = /mage get all from pocket%;/mage give surcoat to %HealerName%;/mage give surcoat to %ThugName%;/all remove cloak 2%;/all wear surcoat%;/all keep surcoat%;/mage pap
/def uncloak = 

/def followAll = /mage follow %ThugName%;/healer follow %ThugName%;/repeat -wTank -1 1 follow %ThugName

/def donate = keep all%; /repeat -S 10 get all from %container%%; donate all
/def kib = /healer locate kren thief

;; Try to handle a disconnect by sending everyone else back to the cath
;; Reconnect works well; getting everyone back together again is iffy at best
/def -h'DISCONNECT' disconnectHook = \
	/echo Lost connection to %1, reconnecting in 5 seconds%; \
	/set dcLoc%1 %mpCurLoc%; \
	/if (botting) \
		/if (%1 =~ 'Thug') \
			/set tmpLoc %mpCurLoc%; \
			/repeat -wHealer -S 1 /goto cath from %%tmpLoc%%; /where %%tmpLoc%; \
			/repeat -wMage -S 1 /goto cath from %%tmpLoc%%; /where %%tmpLoc%; \
			/repeat -wTank -S 1 /goto cath from %%tmpLoc%%; /where %%tmpLoc%; \
		/else \
			/repeat -wThug -S 1 /goto cath%; \
		/endif%; \
	/endif%; \
	/set botting 0%; \
	/repeat -5 1 /world %1
/def -h'CONFAIL' confailHook = /echo Failed to connect to %1, retrying in 5 seconds%;/repeat -5 1 /world %1
/def -hCONNECT connectHook = \
	/if /test dcLoc%1 !~ ''%; /then \
		/repeat -w%1 -4 1 \
			/let tmpLoc %mpCurLoc%%; \
			/goto cath from %{dcLoc%1}%%; \
			/unset dcLoc%1%%; \
			/where %%tmpLoc%%; \
			/world thug%; \
	/endif
